@@ -53,7 +53,16 @@ class AliasedGroup(HelpColorsGroup):
         "list",
         "notation",
         "update",
-        "version"
+        "version",
+        "password",
+        "template",
+        "add",
+        "editor",
+        "field",
+        "record",
+        "file",
+        "cache",
+        "record-type-dir"
     ]
 
     alias_commands = {
@@ -70,6 +79,7 @@ class AliasedGroup(HelpColorsGroup):
         "u": "update",
         "v": "version",
         "exit": "quit",
+        "pass": "password",
         "q": "quit",
         "help": "--help"
     }
@@ -191,7 +201,7 @@ def base_command_help(f):
             update = update_available("keeper-secrets-manager-core", versions)
             if update is not None:
                 doc += spacer + "Version {} is available for the SDK".format(update.available_version)
-    except Exception as _:
+    except (Exception,):
         pass
 
     f.__doc__ = doc
@@ -234,7 +244,7 @@ def cli(ctx, ini_file, profile_name, output, color, cache):
     help_options_color='green'
 )
 def profile_command():
-    """Commands for profile management."""
+    """Manage local client device profiles"""
     pass
 
 
@@ -250,12 +260,12 @@ def profile_command():
 @click.argument('token-arg', type=str, nargs=-1)
 @click.pass_context
 def profile_init_command(ctx, token, hostname, ini_file, profile_name, token_arg):
-    """Initialize a profile."""
+    """Initialize a profile"""
 
     if token is None and len(token_arg) > 0:
         token = token_arg[0]
     if token is None:
-        sys.exit("An one time access token is required either as a command parameter or an argument.")
+        raise KsmCliException("A one time access token is required either as a command parameter or an argument.")
 
     # Since the top level commands are available for all command, it might be confusing the init command since
     # it take
@@ -279,7 +289,7 @@ def profile_init_command(ctx, token, hostname, ini_file, profile_name, token_arg
 @click.option('--json', is_flag=True, help='Return secret as JSON')
 @click.pass_context
 def profile_list_command(ctx, json):
-    """List all profiles."""
+    """List all profiles"""
 
     output = "text"
     if json is True:
@@ -296,7 +306,7 @@ def profile_list_command(ctx, json):
 @click.argument('profile-name', type=str, required=True, nargs=1)
 @click.pass_context
 def profile_active_command(ctx, profile_name):
-    """Set the active profile."""
+    """Set the active profile"""
     Profile(cli=ctx.obj["cli"]).set_active(
         profile_name=profile_name
     )
@@ -313,7 +323,7 @@ def profile_active_command(ctx, profile_name):
 @click.argument('profile-name', type=str, required=False, nargs=1)
 @click.pass_context
 def profile_export_command(ctx, plain, file_format, profile_name):
-    """Create a new config file from a profile."""
+    """Create a new config file from a profile"""
     Profile(cli=ctx.obj["cli"]).export_config(
         plain=plain,
         file_format=file_format,
@@ -330,7 +340,7 @@ def profile_export_command(ctx, plain, file_format, profile_name):
 @click.argument('config-base64', type=str, required=True, nargs=1)
 @click.pass_context
 def profile_import_command(ctx, output_file, config_base64):
-    """Import an encrypted config file."""
+    """Import an encrypted config file"""
     Profile(cli=ctx.obj["cli"]).import_config(
         file=output_file,
         config_base64=config_base64
@@ -355,7 +365,7 @@ profile_command.add_command(profile_import_command)
 )
 @click.pass_context
 def secret_command(ctx):
-    """Commands for secrets."""
+    """Commands for secrets"""
     ctx.obj["secret"] = Secret(cli=ctx.obj["cli"])
 
 
@@ -368,7 +378,7 @@ def secret_command(ctx):
 @click.option('--json', is_flag=True, help='Return secret as JSON')
 @click.pass_context
 def secret_list_command(ctx, uid, json):
-    """List all secrets."""
+    """List all secrets"""
 
     output = "text"
     if json is True:
@@ -397,7 +407,7 @@ def secret_list_command(ctx, uid, json):
 @click.argument('extra-uid', type=str, nargs=-1)
 @click.pass_context
 def secret_get_command(ctx, uid, title, field, query, json, force_array, unmask, inflate, extra_uid):
-    """Get secret record(s)."""
+    """Get secret record(s)"""
 
     uid_list = []
     if uid is not None:
@@ -414,10 +424,10 @@ def secret_get_command(ctx, uid, title, field, query, json, force_array, unmask,
     total_query = len(uid_list) + len(title)
 
     if total_query == 0:
-        raise Exception("No uid or title specified for secret get command.")
+        raise KsmCliException("No uid or title specified for secret get command.")
 
     if total_query > 1 and field is not None:
-        raise Exception("Cannot perform field search on multiple records. Only choose one uid/title.")
+        raise KsmCliException("Cannot perform field search on multiple records. Only choose one uid/title.")
 
     ctx.obj["secret"].query(
         uids=uid_list,
@@ -441,7 +451,7 @@ def secret_get_command(ctx, uid, title, field, query, json, force_array, unmask,
 @click.argument('text', type=str, nargs=1)
 @click.pass_context
 def secret_notation_command(ctx, text):
-    """Get secret record via notation."""
+    """Get secret record via notation"""
     ctx.obj["secret"].get_via_notation(notation=text)
 
 
@@ -458,7 +468,7 @@ def secret_notation_command(ctx, text):
               help="Update value in custom field section of vault using JSON")
 @click.pass_context
 def secret_update_command(ctx, uid, field, custom_field, field_json, custom_field_json):
-    """Update an existing record."""
+    """Update an existing record"""
     ctx.obj["secret"].update(
         uid=uid,
         fields=field,
@@ -480,7 +490,7 @@ def secret_update_command(ctx, uid, field, custom_field, field_json, custom_fiel
 @click.option('--create-folders', is_flag=True, help='Create folder for filename path.')
 @click.pass_context
 def secret_download_command(ctx, uid, name, file_output, create_folders):
-    """Download a file from a secret record."""
+    """Download a file from a secret record"""
     ctx.obj["secret"].download(
         uid=uid,
         name=name,
@@ -497,18 +507,238 @@ def secret_download_command(ctx, uid, name, file_output, create_folders):
 @click.argument('uid', type=str, nargs=1)
 @click.pass_context
 def secret_totp_command(ctx, uid):
-    """Get TOTP code from a secret Record UID."""
+    """Get TOTP code from a secret Record UID"""
     ctx.obj["secret"].get_totp_code(
         uid=uid
     )
 
 
+@click.command(
+    name='password',
+    cls=HelpColorsCommand,
+    help_options_color='blue'
+)
+@click.pass_context
+@click.option('--length', '-l', required=False, type=int, default=0,
+              help="Length of password. Will evenly split lowercase, uppercase, digits, and special characters.")
+@click.option('-lc', required=False, type=int, default=0, help="Number of lowercase letters.")
+@click.option('-uc', required=False, type=int, default=0, help="Number of uppercase letters.")
+@click.option('-d', required=False, type=int, default=0, help="Number of digits.")
+@click.option('-sc', required=False, type=int, default=0, help="Number of special characters.")
+def secret_password_command(ctx, length, lc, uc, d, sc):
+    """Generate a password"""
+
+    # Get the total length of character groups. If the total is greater than 0,
+    # then make the length the total count of the character groups.
+    char_count = lc + uc + d + sc
+    if char_count > 0:
+        if 0 < length != char_count:
+            raise KsmCliException("Both length and at least one of the character group counts has been set. "
+                                  "Either set the length or the character group counts, but not both. The total count "
+                                  "of the character groups counts will determine the length.")
+        length = char_count
+
+    if length == 0:
+        length = 64
+
+    ctx.obj["secret"].generate_password(
+        length=length,
+        lowercase=lc,
+        uppercase=uc,
+        digits=d,
+        special_characters=sc
+    )
+
+
+# SECRET TEMPLATE COMMAND
+
+
+@click.group(
+    name='template',
+    cls=AliasedGroup,
+    help_headers_color='yellow',
+    help_options_color='green'
+)
+def secret_template_command():
+    """Record and field information"""
+
+
+@click.command(
+    name='record',
+    cls=HelpColorsCommand,
+    help_options_color='blue'
+)
+@click.pass_context
+@click.option('--show-list', '-l', is_flag=True, help='List available record types.')
+@click.option('--output-format', '-o', type=click.Choice(['yaml', 'json'], case_sensitive=False), default='json',
+              help='File format to export.')
+@click.option('--output-file', '-f', type=str, help='Write template to a file.')
+@click.option('--version', type=click.Choice(['v3'], case_sensitive=False), default='v3',
+              help='Record version.')
+@click.argument('record_type', type=str, nargs=-1)
+def secret_template_record_command(ctx, show_list, output_format, output_file, version, record_type):
+    """Get a record type or list available record types"""
+
+    if show_list is True:
+        print("", file=sys.stderr)
+        ctx.obj["secret"].get_record_type_list(version=version)
+        print("", file=sys.stderr)
+    else:
+        if record_type is None or len(record_type) == 0:
+            raise KsmCliException("A record type is required.")
+
+        print("", file=sys.stderr)
+        ctx.obj["secret"].get_record_type_template(
+            record_type=record_type[0],
+            version=version,
+            output_format=output_format,
+            file=output_file
+        )
+        print("", file=sys.stderr)
+
+
+@click.command(
+    name='field',
+    cls=HelpColorsCommand,
+    help_options_color='blue'
+)
+@click.pass_context
+@click.option('--show-list', '-l', is_flag=True, help='List available fields types.')
+@click.option('--output-format', '-o', type=click.Choice(['yaml', 'json'], case_sensitive=False), default='json',
+              help='Display field schema in this format.')
+@click.option('--version', type=click.Choice(['v3'], case_sensitive=False), default='v3',
+              help='Record version.')
+@click.argument('field_type', type=str, nargs=-1)
+def secret_template_field_command(ctx, show_list, output_format, version, field_type):
+    """List field types and field schemas"""
+    if show_list is True:
+        print("", file=sys.stderr)
+        ctx.obj["secret"].get_field_type_list(version=version)
+    else:
+        if field_type is None or len(field_type) == 0:
+            raise KsmCliException("A field type is required.")
+
+        print("", file=sys.stderr)
+        ctx.obj["secret"].get_field_type_schema(
+            field_type=field_type[0],
+            output_format=output_format,
+            version=version
+        )
+        print("", file=sys.stderr)
+
+
+secret_template_command.add_command(secret_template_record_command)
+secret_template_command.add_command(secret_template_field_command)
+
+
+# SECRET ADD COMMAND
+
+
+@click.group(
+    name='add',
+    cls=AliasedGroup,
+    help_headers_color='yellow',
+    help_options_color='green'
+)
+def secret_add_command():
+    """Add a secret record to a folder"""
+
+
+@click.command(
+    name='editor',
+    cls=HelpColorsCommand,
+    help_options_color='blue'
+)
+@click.pass_context
+@click.option('--shared-folder-uid', '--sf', required=True, type=str, help="Place record in folder with UID.")
+@click.option('--record-type', '--rt', required=True, type=str, help="Record type")
+@click.option('--password-generate', '-p', is_flag=True, help='Generate passwords for empty password fields.')
+@click.option('--title', '-t', type=str, help="Record title")
+@click.option('--notes', '-n', type=str, help="Record simple note")
+@click.option('--output-format', '-o', type=click.Choice(['yaml', 'json'], case_sensitive=False), default='json',
+              help='File format to display in editor.')
+@click.option('--editor', '-e', type=str, help='Application to use to edit record data.')
+@click.option('--version', type=click.Choice(['v3'], case_sensitive=False), default='v3', help='Record version.')
+def secret_add_editor_command(ctx, shared_folder_uid, record_type, password_generate, title, notes,
+                              output_format, editor, version):
+    """Add a secret record via a text editor"""
+
+    ctx.obj["secret"].add_record_interactive(
+        version=version,
+        folder_uid=shared_folder_uid,
+        record_type=record_type,
+        output_format=output_format,
+        password_generate_flag=password_generate,
+        title=title,
+        notes=notes,
+        editor=editor
+    )
+    print("", file=sys.stderr)
+
+
+@click.command(
+    name='file',
+    cls=HelpColorsCommand,
+    help_options_color='blue'
+)
+@click.pass_context
+@click.option('--shared-folder-uid', '--sf', required=True, type=str, help="Place record in folder with UID.")
+@click.option('--file', '-f', required=True, type=str, help='Add records from record script file.')
+@click.option('--password-generate', '-p', is_flag=True, help='Generate passwords for empty password fields.')
+def secret_add_file_command(ctx, shared_folder_uid, file, password_generate):
+    """Add a secret record(s) from a file"""
+
+    ctx.obj["secret"].add_record_from_file(
+        folder_uid=shared_folder_uid,
+        file=file,
+        password_generate_flag=password_generate,
+    )
+    print("", file=sys.stderr)
+
+
+@click.command(
+    name='field',
+    cls=HelpColorsCommand,
+    help_options_color='blue'
+)
+@click.pass_context
+@click.option('--shared-folder-uid', '--sf', required=True, type=str, help="Place record in folder with UID.")
+@click.option('--record-type', '--rt', required=True, type=str, help="Record type")
+@click.option('--title', '-t', required=True, type=str, help="Record title")
+@click.option('--password-generate', '-p', is_flag=True, help='Generate passwords for empty password fields.')
+@click.option('--notes', '-n', type=str, help="Record simple note")
+@click.option('--version', type=click.Choice(['v3'], case_sensitive=False), default='v3', help='Record version.')
+@click.argument('field_args', type=str, nargs=-1)
+def secret_add_field_command(ctx, shared_folder_uid, record_type, title, password_generate, notes, version,
+                             field_args):
+    """Add a secret record from a command line field arguments"""
+
+    ctx.obj["secret"].add_record_from_field_args(
+        version=version,
+        folder_uid=shared_folder_uid,
+        password_generate_flag=password_generate,
+        record_type=record_type,
+        title=title,
+        notes=notes,
+        field_args=list(field_args)
+    )
+    print("", file=sys.stderr)
+
+
+secret_add_command.add_command(secret_add_field_command)
+secret_add_command.add_command(secret_add_file_command)
+secret_add_command.add_command(secret_add_editor_command)
+
+
 secret_command.add_command(secret_list_command)
 secret_command.add_command(secret_get_command)
 secret_command.add_command(secret_notation_command)
 secret_command.add_command(secret_update_command)
+secret_command.add_command(secret_add_command)
 secret_command.add_command(secret_download_command)
 secret_command.add_command(secret_totp_command)
+secret_command.add_command(secret_password_command)
+secret_command.add_command(secret_template_command)
 
 
 # EXEC COMMAND
@@ -524,7 +754,7 @@ secret_command.add_command(secret_totp_command)
 @click.argument('cmd', type=str, nargs=-1)
 @click.pass_context
 def exec_command(ctx, capture_output, inline, cmd):
-    """Wrap an application and expose secrets in environmental variables."""
+    """Wrap an application and replace env variables"""
     ex = Exec(cli=ctx.obj["cli"])
     ex.execute(cmd=cmd, capture_output=capture_output, inline=inline)
 
@@ -538,7 +768,7 @@ def exec_command(ctx, capture_output, inline, cmd):
 )
 @click.pass_context
 def config_command(ctx):
-    """Configure the command line tool."""
+    """Configure the command line tool"""
     ctx.obj["profile"] = Profile(cli=ctx.obj["cli"])
     pass
 
@@ -550,7 +780,7 @@ def config_command(ctx):
 )
 @click.pass_context
 def config_show_command(ctx):
-    """Show current configuration."""
+    """Show current configuration"""
     ctx.obj["profile"].show_config()
 
 
@@ -567,20 +797,64 @@ def config_log_command(ctx, enable):
 
 
 @click.command(
-    name='record-cache',
+    name='cache',
     cls=HelpColorsCommand,
     help_options_color='blue'
 )
 @click.option('--enable/--disable', required=True, help="Enable or disable cache.")
 @click.pass_context
 def config_cache_command(ctx, enable):
-    """Enable or disable cache"""
+    """Enable or disable record cache"""
     ctx.obj["profile"].set_cache(enable)
 
 
+@click.command(
+    name='record-type-dir',
+    cls=HelpColorsCommand,
+    help_options_color='blue'
+)
+@click.option('--directory', "-d", type=str, help='Location of record type schema directory')
+@click.option('--clear', is_flag=True, help='Clear location of record type schema directory')
+@click.pass_context
+def config_rt_dir_command(ctx, directory, clear):
+    """Set the directory that contains record type schemas"""
+
+    if clear is True:
+        directory = None
+    elif directory is None:
+        raise KsmCliException("Either a --directory is required or the --clear flag set")
+
+    ctx.obj["profile"].set_record_type_dir(directory)
+
+
+@click.command(
+    name='editor',
+    cls=HelpColorsCommand,
+    help_options_color='blue'
+)
+@click.option('--application', "--app", type=str, help='Application path and name to use for editor.')
+@click.option('--blocking', is_flag=True, help='Application requires blocking.')
+@click.option('--process-name', type=str, help='Application process name.')
+@click.option('--clear', is_flag=True, help='Clear location of record type schema directory')
+@click.pass_context
+def config_editor_command(ctx, application, blocking, process_name, clear):
+    """Set the editor to use for record editing"""
+
+    if clear is True:
+        application = None
+        blocking = False
+        process_name = None
+    if clear is not True and application is None:
+        raise KsmCliException("Either a --application is required or the --clear flag set")
+
+    ctx.obj["profile"].set_editor(editor=application, use_blocking=blocking, process_name=process_name)
+
+
 config_command.add_command(config_show_command)
 config_command.add_command(config_log_command)
 config_command.add_command(config_cache_command)
+config_command.add_command(config_rt_dir_command)
+config_command.add_command(config_editor_command)
 
 
 # REDEEM COMMAND
@@ -592,7 +866,7 @@ config_command.add_command(config_cache_command)
 )
 @click.pass_context
 def init_command(ctx):
-    """Redeem an one time access token."""
+    """Initialize a configuration file for integrations"""
     ctx.obj["profile"] = Profile(cli=ctx.obj["cli"])
 
 
@@ -610,7 +884,7 @@ def init_command(ctx):
 @click.argument('token', type=str, nargs=1)
 @click.pass_context
 def init_k8s_command(ctx, name, namespace, hostname, apply, immutable, skip_ssl_verify, token):
-    """Output the config as a k8s secret."""
+    """Output the config as a k8s secret"""
     Init(cli=ctx.obj["cli"], token=token, hostname=hostname, skip_ssl_verify=skip_ssl_verify).get_k8s(
         name=name,
         namespace=namespace,
@@ -630,7 +904,7 @@ def init_k8s_command(ctx, name, namespace, hostname, apply, immutable, skip_ssl_
 @click.argument('token', type=str, nargs=1)
 @click.pass_context
 def init_json_command(ctx, plain, hostname, skip_ssl_verify, token):
-    """Output the config as the default JSON."""
+    """Output the config as base64 encoded JSON"""
     Init(cli=ctx.obj["cli"], token=token, hostname=hostname, skip_ssl_verify=skip_ssl_verify).get_json(plain)
 
 
@@ -645,7 +919,7 @@ init_command.add_command(init_k8s_command)
 )
 @click.pass_context
 def version_command(ctx):
-    """Get module versions and information."""
+    """Get module versions and information"""
 
     versions = get_versions()
 
@@ -671,7 +945,7 @@ def version_command(ctx):
             update = update_available("keeper-secrets-manager-core", versions)
             if update is not None:
                 print("Version {} is available for the SDK".format(update.available_version))
-    except Exception as _:
+    except (Exception,) as _:
         pass
 
 
@@ -680,8 +954,7 @@ def version_command(ctx):
     cls=HelpColorsCommand,
     help_options_color='blue'
 )
-@click.pass_context
-def shell_command(ctx):
+def shell_command():
     """Run KSM in a shell"""
 
     # https://manytools.org/hacker-tools/ascii-banner/
